@@ -4,12 +4,12 @@
 ;;		 Viper Is also a Package for Emacs Rebels.
 
 ;; Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002,
-;;   2003, 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
+;;   2003, 2004, 2005, 2006, 2007, 2008 Free Software Foundation, Inc.
 
 ;; Author: Michael Kifer <kifer@cs.stonybrook.edu>
 ;; Keywords: emulations
 
-(defconst viper-version "3.14 of August 18, 2007"
+(defconst viper-version "3.14 of January 09, 2008"
   "The current version of Viper")
 
 ;; This file is part of GNU Emacs.
@@ -297,28 +297,15 @@
 
 ;;; Code:
 
-(require 'advice)
-(require 'ring)
-
 ;; compiler pacifier
 (defvar mark-even-if-inactive)
 (defvar quail-mode)
 (defvar viper-expert-level)
 (defvar viper-mode-string)
 (defvar viper-major-mode-modifier-list)
-
-;; loading happens only in non-interactive compilation
-;; in order to spare non-viperized emacs from being viperized
-(if noninteractive
-    (eval-when-compile
-      (let ((load-path (cons (expand-file-name ".") load-path)))
-	(or (featurep 'viper-init)
-	    (load "viper-init.el" nil nil 'nosuffix))
-	(or (featurep 'viper-cmd)
-	    (load "viper-cmd.el" nil nil 'nosuffix))
-	)))
 ;; end pacifier
 
+(require 'advice)
 (require 'viper-init)
 (require 'viper-keym)
 
@@ -416,7 +403,7 @@ widget."
   :group 'viper-misc)
 
 (defcustom viper-emacs-state-mode-list
-  '(custom-mode
+  '(Custom-mode
 
     dired-mode
     efs-mode
@@ -610,13 +597,14 @@ This startup message appears whenever you load Viper, unless you type `y' now."
 		    ))
 	      (viper-set-expert-level 'dont-change-unless)))
 
-	(if (eq major-mode 'viper-mode)
-	    (setq major-mode 'fundamental-mode))
-
 	(or (memq major-mode viper-emacs-state-mode-list) ; don't switch to Vi
 	    (memq major-mode viper-insert-state-mode-list) ; don't switch
 	    (viper-change-state-to-vi))
-	)))
+	))
+
+  (if (eq major-mode 'viper-mode)
+      (setq major-mode 'fundamental-mode))
+  )
 
 
 ;; Apply a little heuristic to invoke vi state on major-modes
@@ -690,7 +678,7 @@ It also can't undo some Viper settings."
   (setq default-major-mode
 	(viper-standard-value 'default-major-mode viper-saved-non-viper-variables))
 
-  (if viper-emacs-p
+  (if (featurep 'emacs)
       (setq-default
        mark-even-if-inactive
        (viper-standard-value
@@ -701,7 +689,7 @@ It also can't undo some Viper settings."
       (and (fboundp 'add-to-ordered-list) (boundp 'emulation-mode-map-alists))
     (viper-delocalize-var 'minor-mode-map-alist))
   (viper-delocalize-var 'require-final-newline)
-  (if viper-xemacs-p (viper-delocalize-var 'bar-cursor))
+  (if (featurep 'xemacs) (viper-delocalize-var 'bar-cursor))
 
 
   ;; deactivate all advices done by Viper.
@@ -788,7 +776,7 @@ It also can't undo some Viper settings."
   ;; In emacs, we have to advice handle-switch-frame
   ;; This advice is undone earlier, when all advices matchine "viper-" are
   ;; deactivated.
-  (if viper-xemacs-p
+  (if (featurep 'xemacs)
       (remove-hook 'mouse-leave-frame-hook 'viper-remember-current-frame))
   ) ; end viper-go-away
 
@@ -870,7 +858,9 @@ It also can't undo some Viper settings."
 	       (modify-frame-parameters
 		(selected-frame)
 		(list (cons 'viper-vi-state-cursor-color
-			    (viper-get-cursor-color))))))
+			    (viper-get-cursor-color))))
+	       (setq viper-vi-state-cursor-color (viper-get-cursor-color))
+	       ))
 
   ;; Tell vc-diff to put *vc* in Vi mode
   (if (featurep 'vc)
@@ -913,6 +903,7 @@ It also can't undo some Viper settings."
     (modify-frame-parameters
 	(selected-frame)
 	(list (cons 'viper-vi-state-cursor-color (ad-get-arg 0))))
+    (setq viper-vi-state-cursor-color (ad-get-arg 0))
     )
 
   (when (and (fboundp 'add-to-ordered-list) (boundp 'emulation-mode-map-alists))
@@ -981,7 +972,7 @@ It also can't undo some Viper settings."
 	)))
 
   ;; International input methods
-  (if viper-emacs-p
+  (if (featurep 'emacs)
       (eval-after-load "mule-cmds"
 	'(progn
 	   (defadvice inactivate-input-method (after viper-mule-advice activate)
@@ -1022,7 +1013,7 @@ It also can't undo some Viper settings."
 	require-final-newline t)
 
   ;; don't bark when mark is inactive
-  (if viper-emacs-p
+  (if (featurep 'emacs)
       (setq mark-even-if-inactive t))
 
   (setq scroll-step 1)
@@ -1032,74 +1023,74 @@ It also can't undo some Viper settings."
       (setq global-mode-string
 	    (append '("" viper-mode-string) (cdr global-mode-string))))
 
-  (viper-cond-compile-for-xemacs-or-emacs
-   ;; XEmacs
-   (defadvice describe-key (before viper-describe-key-ad protect activate)
-     "Force to read key via `viper-read-key-sequence'."
-     (interactive (list (viper-read-key-sequence "Describe key: "))))
-   ;; Emacs
-   (defadvice describe-key (before viper-describe-key-ad protect activate)
-     "Force to read key via `viper-read-key-sequence'."
-     (interactive (let (key)
-		    (setq key (viper-read-key-sequence
-			       "Describe key (or click or menu item): "))
-		    (list key
-			  (prefix-numeric-value current-prefix-arg)
-			  ;; If KEY is a down-event, read also the
-			  ;; corresponding up-event.
-			  (and (vectorp key)
-			       (let ((last-idx (1- (length key))))
-				 (and (eventp (aref key last-idx))
-				      (memq 'down (event-modifiers
-						   (aref key last-idx)))))
-			       (or (and (eventp (aref key 0))
-					(memq 'down (event-modifiers
-						     (aref key 0)))
-					;; For the C-down-mouse-2 popup menu,
-					;; there is no subsequent up-event
-					(= (length key) 1))
-				   (and (> (length key) 1)
-					(eventp (aref key 1))
-					(memq 'down (event-modifiers (aref key 1)))))
-			       (read-event))))))
-   ) ; viper-cond-compile-for-xemacs-or-emacs
-
-  (viper-cond-compile-for-xemacs-or-emacs
-   ;; XEmacs
-   (defadvice describe-key-briefly
-     (before viper-describe-key-briefly-ad protect activate)
-     "Force to read key via `viper-read-key-sequence'."
-     (interactive (list (viper-read-key-sequence "Describe key briefly: "))))
-   ;; Emacs
-   (defadvice describe-key-briefly
-     (before viper-describe-key-briefly-ad protect activate)
-     "Force to read key via `viper-read-key-sequence'."
-     (interactive (let (key)
-		    (setq key (viper-read-key-sequence
-			       "Describe key (or click or menu item): "))
-		    ;; If KEY is a down-event, read and discard the
-		    ;; corresponding up-event.
-		    (and (vectorp key)
-			 (let ((last-idx (1- (length key))))
-			   (and (eventp (aref key last-idx))
-				(memq 'down (event-modifiers (aref key last-idx)))))
-			 (read-event))
-		    (list key
-			  (if current-prefix-arg
-			      (prefix-numeric-value current-prefix-arg))
-			  1))))
-   ) ;; viper-cond-compile-for-xemacs-or-emacs
-
+  (if (featurep 'xemacs)
+      ;; XEmacs
+      (defadvice describe-key (before viper-describe-key-ad protect activate)
+	"Force to read key via `viper-read-key-sequence'."
+	(interactive (list (viper-read-key-sequence "Describe key: "))))
+    ;; Emacs
+    (defadvice describe-key (before viper-describe-key-ad protect activate)
+      "Force to read key via `viper-read-key-sequence'."
+      (interactive (let (key)
+		     (setq key (viper-read-key-sequence
+				"Describe key (or click or menu item): "))
+		     (list key
+			   (prefix-numeric-value current-prefix-arg)
+			   ;; If KEY is a down-event, read also the
+			   ;; corresponding up-event.
+			   (and (vectorp key)
+				(let ((last-idx (1- (length key))))
+				  (and (eventp (aref key last-idx))
+				       (memq 'down (event-modifiers
+						    (aref key last-idx)))))
+				(or (and (eventp (aref key 0))
+					 (memq 'down (event-modifiers
+						      (aref key 0)))
+					 ;; For the C-down-mouse-2 popup menu,
+					 ;; there is no subsequent up-event
+					 (= (length key) 1))
+				    (and (> (length key) 1)
+					 (eventp (aref key 1))
+					 (memq 'down (event-modifiers (aref key 1)))))
+				(read-event))))))
+    ) ; (if (featurep 'xemacs)
+  
+  (if (featurep 'xemacs)
+      ;; XEmacs
+      (defadvice describe-key-briefly
+	(before viper-describe-key-briefly-ad protect activate)
+	"Force to read key via `viper-read-key-sequence'."
+	(interactive (list (viper-read-key-sequence "Describe key briefly: "))))
+    ;; Emacs
+    (defadvice describe-key-briefly
+      (before viper-describe-key-briefly-ad protect activate)
+      "Force to read key via `viper-read-key-sequence'."
+      (interactive (let (key)
+		     (setq key (viper-read-key-sequence
+				"Describe key (or click or menu item): "))
+		     ;; If KEY is a down-event, read and discard the
+		     ;; corresponding up-event.
+		     (and (vectorp key)
+			  (let ((last-idx (1- (length key))))
+			    (and (eventp (aref key last-idx))
+				 (memq 'down (event-modifiers (aref key last-idx)))))
+			  (read-event))
+		     (list key
+			   (if current-prefix-arg
+			       (prefix-numeric-value current-prefix-arg))
+			   1))))
+    ) ; (if (featurep 'xemacs)
+  
   (defadvice find-file (before viper-add-suffix-advice activate)
     "Use `read-file-name' for reading arguments."
     (interactive (cons (read-file-name "Find file: " nil default-directory)
 		       ;; XEmacs: if Mule & prefix arg, ask for coding system
-		       (cond ((and viper-xemacs-p (featurep 'mule))
+		       (cond ((and (featurep 'xemacs) (featurep 'mule))
 			      (list
 			       (and current-prefix-arg
 				    (read-coding-system "Coding-system: "))))
 			     ;; Emacs: do wildcards
-			     ((and viper-emacs-p (boundp 'find-file-wildcards))
+			     ((and (featurep 'emacs) (boundp 'find-file-wildcards))
 				   (list find-file-wildcards))))
 		 ))
 
@@ -1108,12 +1099,12 @@ It also can't undo some Viper settings."
     (interactive (cons (read-file-name "Find file in other window: "
 				       nil default-directory)
 		       ;; XEmacs: if Mule & prefix arg, ask for coding system
-		       (cond ((and viper-xemacs-p (featurep 'mule))
+		       (cond ((and (featurep 'xemacs) (featurep 'mule))
 			      (list
 			       (and current-prefix-arg
 				    (read-coding-system "Coding-system: "))))
 			     ;; Emacs: do wildcards
-			     ((and viper-emacs-p (boundp 'find-file-wildcards))
+			     ((and (featurep 'emacs) (boundp 'find-file-wildcards))
 			      (list find-file-wildcards))))
 		 ))
 
@@ -1123,12 +1114,12 @@ It also can't undo some Viper settings."
     (interactive (cons (read-file-name "Find file in other frame: "
 				       nil default-directory)
 		       ;; XEmacs: if Mule & prefix arg, ask for coding system
-		       (cond ((and viper-xemacs-p (featurep 'mule))
+		       (cond ((and (featurep 'xemacs) (featurep 'mule))
 			      (list
 			       (and current-prefix-arg
 				    (read-coding-system "Coding-system: "))))
 			     ;; Emacs: do wildcards
-			     ((and viper-emacs-p (boundp 'find-file-wildcards))
+			     ((and (featurep 'emacs) (boundp 'find-file-wildcards))
 			      (list find-file-wildcards))))
 		 ))
 
@@ -1159,7 +1150,7 @@ It also can't undo some Viper settings."
 
   ;; catch frame switching event
   (if (viper-window-display-p)
-      (if viper-xemacs-p
+      (if (featurep 'xemacs)
 	     (add-hook 'mouse-leave-frame-hook
 		       'viper-remember-current-frame)
 	   (defadvice handle-switch-frame (before viper-frame-advice activate)
@@ -1227,7 +1218,7 @@ These two lines must come in the order given.
 	   (cons 'mode-line-buffer-identification
 		 (list (default-value 'mode-line-buffer-identification)))
 	   (cons 'global-mode-string (list global-mode-string))
-	   (if viper-emacs-p
+	   (if (featurep 'emacs)
 	       (cons 'mark-even-if-inactive (list mark-even-if-inactive)))
 	   )))
 
